{==============================================================================}
{ DSPLab                                                                       }
{ Copyright 2003 Tarasov Eugen                                                 }
{                                                                              }
{ Implementation of simple frequency meter                                     }
{                                                                              }
{ Last modified January, 2003                                                  }
{                                                                              }
{ e-mail: dsplab@teworks.com                                                   }
{ http://www.teworks.com                                                       }
{==============================================================================}

unit FreqMeter;

interface

uses Classes, Windows, dspTypes, dspIIRFilters;

type
  TFrequencyMeter = class(TComponent)
  private
    FFilter: TdspIIRFilter;
    FFrequency: Single;
    FSampleRate: Single;
    FThreshold: Single;

  protected
    PrevCounter: Integer;
    Counter: Integer;
    PrevValue: Single;
    WasFront: Boolean;

  public
    constructor Create(AOwner: TComponent); override;

    procedure Reset;
    procedure Put(Value: Single);
    property Frequency: Single read FFrequency;

  published
    property Threshold: Single read FThreshold write FThreshold;
    property SampleRate: Single read FSampleRate write FSampleRate;
  end;

implementation


constructor TFrequencyMeter.Create(AOwner: TComponent);
begin
  inherited;
  FFilter:= TdspIIRFilter.Create(Self);
  FFilter.Kind:= fkButterworth;
  FFilter.Response:= ftHighpass;
  FFilter.Order:= 1;
  FFilter.Frequency1:= 1;

  FSampleRate:= 44100; // by default
  FThreshold:= 1;
end;


procedure TFrequencyMeter.Reset;
begin
  PrevCounter:= 0;
  Counter:= 0;
  FFRequency:= 0;
  WasFront:= false;
end;


procedure TFrequencyMeter.Put(Value: Single);
begin
  Inc(Counter);
  Value:= FFilter.Filter(Value);
  if Value > 0 then
  begin
    if not WasFront then
    begin
      FFrequency:= FSampleRate / (Counter - PrevCounter);
      PrevCounter:= Counter;
      WasFront:= true;
    end;
  end else WasFront:= false;
end;


end.
